package com.hw.admin.model.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.hw.admin.model.domain.WorkPacker;
import com.hw.admin.model.domain.WorkPackerExcel;
import com.hw.admin.model.mapper.WorkPackerMapper;
import com.hw.admin.model.service.WorkPackerService;
import com.hw.admin.system.listener.WorkPackerListener;
import com.hw.admin.system.utils.DateExUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
@Slf4j
public class WorkPackerServiceImpl implements WorkPackerService {

    @Value("${file.path}")
    private String filepath;

    @Autowired
    private WorkPackerMapper workPackerMapper;

    @Override
    public int add(WorkPacker workPacker) {

        List<WorkPackerExcel> list = new ArrayList();
        WorkPackerExcel workPacker1 = new WorkPackerExcel();
        BeanUtils.copyProperties(workPacker, workPacker1);
        for (int i = 0; i < 10000; i++) {
            list.add(workPacker1);
        }
        // 写法2
        String filePaht = filepath + DateExUtils.commonDatePath();
        //不存在创建文件夹
        File file = new File(filePaht);
        if (!file.exists()) {
            file.mkdirs();
        }
        String fileName = filePaht + "simpleWrite" + System.currentTimeMillis() + ".xlsx";
        // 这里 需要指定写用哪个class去写，然后写到第一个sheet，名字为模板 然后文件流会自动关闭
        // 如果这里想使用03 则 传入excelType参数即可
        EasyExcel.write(fileName, WorkPackerExcel.class).sheet("模板").doWrite(list);

        return 0;
        //return workPackerMapper.inserList(list);
    }


    @Override
    public void task(String time) {
        // 有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
        // 写法3：
        String fileName = "D:\\project\\delivery_project\\远程制药\\后台\\ycpharmacy_excel_service\\excel\\包装机数据测试表.xlsx";
        // 这里 需要指定读用哪个class去读，然后读取第一个sheet 文件流会自动关闭
        EasyExcel.read(fileName, WorkPackerExcel.class, new WorkPackerListener(workPackerMapper, fileName)).sheet().doRead();
    }

    @Override
    public void task(String time, String fileName) {
        // 有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
        // 写法3：
        // 这里 需要指定读用哪个class去读，然后读取第一个sheet 文件流会自动关闭
        EasyExcel.read(fileName, WorkPackerExcel.class, new WorkPackerListener(workPackerMapper, fileName)).sheet().doRead();
    }

    @Override
    public List<WorkPacker> list(WorkPacker workPacker) {
        QueryWrapper<WorkPacker> queryWrapper = new QueryWrapper<WorkPacker>();
        queryWrapper.eq("del_flag", 0);
        queryWrapper.orderByDesc("create_time");
        return workPackerMapper.selectList(queryWrapper);
    }

    @Override
    public WorkPacker get(Long id) {
        checkData(id);
        return workPackerMapper.selectById(id);
    }

    @Override
    public int update() {
        Set<String> files = new HashSet<>();
        files.addAll(getFile(filepath));
        for (String fileName : files) {
            QueryWrapper<WorkPacker> queryWrapper = new QueryWrapper<WorkPacker>();
            queryWrapper.eq("del_flag", 0);
            queryWrapper.eq("file_path", fileName);
            Integer count = workPackerMapper.selectCount(queryWrapper);
            if ("0".equals(count.toString())) {
                log.info("存在遗漏数据进行补录:{}",fileName);
                EasyExcel.read(fileName, WorkPackerExcel.class, new WorkPackerListener(workPackerMapper, fileName)).sheet().doRead();
            }
        }
        return 0;
    }

    // 给定目录的绝对路径，获取该目录下的所有文件（子目录的文件也可递归得到）
    public static List<String> getFile(String path) {
        List<String> files = new ArrayList<>();
        // File对象 可以是文件或者目录
        File file = new File(path);
        File[] array = file.listFiles();
        for (int i = 0; i < array.length; i++) {
            if (array[i].isFile()) {
                // only take file name
                //log.info("^^^^^" + array[i].getName());
                // take file path and name
                //log.info("#####" + array[i]);
                // take file path and name
                //log.info("*****" + array[i].getPath());
                files.add(array[i].getPath());
            } else if (array[i].isDirectory()) {
                files.addAll(getFile(array[i].getPath()));
            }
        }
        return files;
    }

    @Override
    public int delete(Long id) {
        checkData(id);
        WorkPacker workPacker = new WorkPacker();
        workPacker.setId(id);
        workPacker.setDelFlag("1");
        return workPackerMapper.updateById(workPacker);
    }

    @Override
    public int deleteByIds(List<Long> ids) {
        return workPackerMapper.deleteBatchIds(ids);
    }

    @Override
    public int inserList(List<WorkPacker> workPacker) {

        return workPackerMapper.inserList(workPacker);
    }

    private WorkPacker checkData(Long id) {
        WorkPacker workPacker = workPackerMapper.selectById(id);
        if (StringUtils.isEmpty(workPacker)) {
            throw new RuntimeException("数据不存在");
        }
        if ("1".equals(workPacker.getDelFlag())) {
            throw new RuntimeException("数据不存在");
        }
        return workPacker;
    }
}
